# -*- coding: UTF-8 -*-

import sys
import base64
import httplib2
import json
import os
from common import config, logger

requestTimeout = 3


class RestState(object):
    """ Статичечкий класс для отправки rest-статуса загрузки справочника/продаж """

    STATUS_ERROR = 'STATUS_ERROR'  # Ошибка загрузки/выгрузки справочника
    STATUS_READY = 'STATUS_READY'  # Справочник успешно загружен/выгружен
    STATUS_READY_WITH_ERROR = 'STATUS_READY_WITH_ERROR'  # Справочник загружен с ошибками
    STATUS_DURING = 'STATUS_DURING'  # Справочник в данный момент загружается/выгружается
    formats = {'aif': 'ARTIXINTERCHANGE', 'shtrih': 'SHTRIHM', 'smag-ukm': 'SUPERMAG', 'smag-ukm4': 'SUPERMAG4',
               'oracle': 'ORACLE', 'atol': 'ATOL', 'soap': 'ARTIXINTERCHANGE_SOAP', 'crystal': 'CRYSTAL',
               'rest': 'ARTIXINTERCHANGE_REST', 'ns2000': 'NS2000', 'pilot': 'PILOT', 'simple_file': 'SIMPLE-FILE'}

    log = logger.getLogger('Service')
    if not config.read(os.path.join(os.path.dirname(os.path.realpath(__file__)), '../../config/%s/exchangers.ini' % logger.location)):
        log.error(u"Configuration file 'exchangers.ini' not found. The system 'exchangers' is not running")
        exit(1)
    cs_host = config.getString('CS', 'node.host', 'localhost')
    cs_port = config.getString('CS', 'node.port', '8080')
    cs_prefix = config.getString('CS', 'node.prefix', '/CSrest/rest')
    cs_login = config.getString('CS', 'login', 'admin')
    cs_password = config.getString('CS', 'passwd', 'admin')
    auth = base64.b64encode('{0}:{1}'.format(cs_login, cs_password).encode('utf-8')).decode()
    REST_URL = 'http://' + cs_host + ':' + cs_port + cs_prefix
    SALES_BACKOFFICE_URL = REST_URL + '/sales/backOfficeState'
    DISTS_BACKOFFICE_URL = REST_URL + '/dicts/backOfficeState'
    SALES_CASHUNLOAD_URL = cs_prefix + '/sales/cashUnloadState'
    jsonHeader = {'Content-Type': 'application/json', "Accept": "application/json", 'Authorization': 'Basic %s' % auth}

    @classmethod
    def sendDictState(cls, node, messageEn, messageRu, status, fileName=None):
        body = {'messageEn': messageEn, 'messageRu': messageRu, 'status': status}
        log = logger.getUploadLogger("Node", node.id)
        url = cls.DISTS_BACKOFFICE_URL + '/%s' % node.id
        # отправка статуса только когда задан statusServer
        if hasattr(node.currentConverter, 'statusServer') and node.currentConverter.statusServer:
            url = url.replace(cls.cs_host + ':' + cls.cs_port, node.currentConverter.statusServer)
            if node.currentConverter.dict_state_id:
                log.debug('UPDATE dictState')
                method = 'PUT'
                url += '/%s' % node.currentConverter.dict_state_id
            else:
                method = 'POST'
                body['format'] = cls.formats.get(node.currentConverter.name, 'UNKNOWN')
                body['fileName'] = cls.__convertFileName(fileName)
            body = json.dumps(body)
            log.info(u"Отправка статуса(URL: '%s', method: '%s', headers: '%s', body: '%s')"
                    % (url, method, RestState.jsonHeader, body))
            try:
                http = httplib2.Http(timeout=requestTimeout)
                response, content = http.request(url, method, headers=RestState.jsonHeader, body=body)
                if response.status == 200:
                    return json.loads(content)
                else:
                    log.warn(u'Ошибка отправки статуса конвертирования справочника на КС. %s' % content)
            except Exception as e:
                log.warn(u'Ошибка отправки статуса конвертирования справочника на КС. %s' % str(e))
            finally:
                if sys.version_info.major == 3:
                    http.close()
        return None

    @classmethod
    def sendSaleState(cls, node, cashCode, shiftNums, messageEn, messageRu, status, format,
            resultFiles=None, cashUnloadStateId=None, statusHost=None, isSendDebugStatus=True):
        log = logger.getUnloadLogger("Node", node.id)
        if logger.location == 'cs':
            body = {'messageEn': messageEn, 'messageRu': messageRu, 'status': status, 'format': format}
            if resultFiles:
                body['resultFiles'] = resultFiles
            results = []
            body = json.dumps(body)
            log.info(u'send saleState - body: %s' % body)
            # Строчка ниже получает cashId кассы, зная код кассы и id магазина. Это можно сделать проще и пробросить его сверху,
            # но на текущий момент опасно менять что-то в exchanger за пределами тела метода, т.к. можно нарваться на внезапные проблемы в production
            cashId = list(filter(lambda x: x.code == cashCode, node.cashes))[0].cashId
            for shiftNum in shiftNums:
                # такой метод должен появиться в КС начиная с версии 3.0.246
                url = cls.SALES_BACKOFFICE_URL + '/append?shopid=%s&cashCode=%s&cashid=%s&shiftNum=%s'\
                        % (node.id, cashCode, cashId, shiftNum)
                result = cls.__sendState(url, 'POST', body, log)
                if result:
                    results.append(result)
            return results
        elif logger.location == 'cash' and cashUnloadStateId and statusHost:
            return cls.__updateCashUnloadStatus(cashUnloadStateId, messageEn, messageRu, status, log,
                    statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
        return None

    @classmethod
    def updateSaleState(cls, node, saleBackOfficeIds, state, cashUnloadStateId=None,
            statusHost=None, isSendDebugStatus=True):
        log = logger.getUnloadLogger("Node", node.id)
        if logger.location == 'cs':
            results = []
            body = json.dumps(state)
            log.debug(u'update saleState - ids: %s body: %s' % (saleBackOfficeIds, body))
            for saleBackOfficeId in saleBackOfficeIds:
                url = cls.SALES_BACKOFFICE_URL + '/%s' % saleBackOfficeId
                result = cls.__sendState(url, 'PUT', body, log)
                if result:
                    results.append(result)
            return results
        elif logger.location == 'cash' and cashUnloadStateId and statusHost:
            return cls.__updateCashUnloadStatus(cashUnloadStateId, state['messageEn'], state['messageRu'],
                    state['status'], log, statusHost=statusHost, isSendDebugStatus=isSendDebugStatus)
        return None

    @staticmethod
    def startLoadDictCash(cashCode, log):
        try:
            http = httplib2.Http(timeout=requestTimeout)
            response, content = http.request(RestState.REST_URL + '/cashes/%s/dicts/startLoad' % cashCode, 'POST',
                    headers=RestState.jsonHeader)
            if response.status == 200:
                log.info(u'Oтправлен сигнал на кассу %s для запуска загрузки справочников.' % cashCode)
            else:
                log.warn(u'Получен некоректный http-код %s во время отправки сигнала на кассу %s для запуска загрузки справочников: %s' % (
                    response.status, cashCode, content))
        except Exception as e:
            log.warn(u'Во время оотправки сигнала на кассу %s для запуска загрузки справочников произошла ошибка: %s' % (cashCode, str(e)))
        finally:
            if sys.version_info.major == 3:
                http.close()

    @staticmethod
    def rereadQueuesInNES(log, nesHost='localhost:28081'):
        try:
            nesHost = nesHost if nesHost else 'localhost:28081'
            http = httplib2.Http(timeout=requestTimeout)
            response, content = http.request('http://%s/nes/queue_reread' % nesHost, 'PUT', headers=RestState.jsonHeader)
            if response.status == 200:
                log.info(u'Oтправлен сигнал в NES, чтобы он перечитал очереди')
            else:
                log.warn(u'Ошибка отправки сигнала в NES, чтобы он перечитал очереди. Он перечитает их позже')
        except:
            log.warn(u'Ошибка отправки сигнала в NES, чтобы он перечитал очереди. Он перечитает их позже')
        finally:
            if sys.version_info.major == 3:
                http.close()

    @staticmethod
    def startLoadNES(log, shopId, nesHost='localhost:28081'):
        try:
            nesHost = nesHost if nesHost else 'localhost:28081'
            http = httplib2.Http(timeout=requestTimeout)
            log.info(u'send signal to nes on host:%s' % nesHost)
            response, content = http.request('http://%s/nes/queue_load' % nesHost, 'PUT', body='{ "shopId":"%s"}' % shopId, headers=RestState.jsonHeader)
            if response.status == 200:
                log.info(u'Oтправлен сигнал в NES, чтобы он приступил к загрузке для магазина:%s' % shopId)
            else:
                log.warn(u'Ошибка отправки сигнала в NES, чтобы он приступил к загрузке для магазина:%s. Он загрузит данные позже' % shopId)
        except:
            log.warn(u'Ошибка отправки сигнала в NES, чтобы он приступил к загрузке для магазина:%s. Он загрузит данные позже' % shopId)
        finally:
            if sys.version_info.major == 3:
                http.close()

    @classmethod
    def getDictState(cls, node, fileName):
        if logger.location == 'cs':
            http = httplib2.Http(timeout=requestTimeout)
            url = cls.DISTS_BACKOFFICE_URL + '/%s?fileName=%s' % (node.id, cls.__convertFileName(fileName))
            response, content = http.request(url, 'GET', headers=RestState.jsonHeader)
            if sys.version_info.major == 3:
                http.close()
            if response.status == 200:
                dictStates = json.loads(content)
                if dictStates and isinstance(dictStates, list):
                    dictStates = [value for value in dictStates if value is not None]
                    if len(dictStates) > 0:
                        return sorted(dictStates, key=lambda k: k['time'], reverse=True)[0]['id']
        return None

    @classmethod
    def __updateCashUnloadStatus(cls, cashUnloadStateId, messageEn, messageRu, status, log, statusHost=None, isSendDebugStatus=True):
        if status == cls.STATUS_READY:
            status = cls.STATUS_DURING
        # Если статусы не надо отправлять в debug режиме, то не отправляем статусы STATUS_DURING
        if status == cls.STATUS_DURING and not isSendDebugStatus:
            return None
        body = json.dumps({'messageEn': messageEn, 'messageRu': messageRu, 'status': status})
        url = 'http://' + statusHost + cls.SALES_CASHUNLOAD_URL + '/%s' % cashUnloadStateId
        return cls.__sendState(url, 'PUT', body, log)

    @staticmethod
    def __sendState(url, method, body, log):
        log.info(u"Отправка статуса(URL: '%s', method: '%s', headers: '%s', body: '%s')" % (url, method, RestState.jsonHeader, body))
        try:
            http = httplib2.Http(timeout=requestTimeout)
            response, content = http.request(url, method, headers=RestState.jsonHeader, body=body)
            if response.status == 200:
                return json.loads(content)
            else:
                log.warn(u'Получен некоректный http-код %s во время отправки статуса на КС: %s' % (response.status, content))
        except Exception as e:
            log.warn(u'Во время отправки статуса на КС произошла ошибка: %s' % str(e))
        finally:
            if sys.version_info.major == 3:
                http.close()
        return None

    @staticmethod
    def __convertFileName(fileName):
        if isinstance(fileName, str):
            return os.path.basename(fileName)
        if isinstance(fileName, list):
            result = []
            for f in fileName:
                result.append(os.path.basename(f))
                return str(result)
        return None
