# -*- coding: UTF-8 -*-

'''
Created on 12.04.2012

@author: kov
'''

import os
from types import *
from upload.converter.smagPlugins.crc16 import *
from common import aif_objects as aif
from common import logger
from upload.inserter import InserterError
from upload.converter.smagPlugins.catalogfactory import CatalogFactory
import csv

log = None
logInfo = "SMAG-UKM4 converter: "
packSize = 99
format = 'dat'
workDir = '/tmp'
charset = 'cp1251'
errorObjectsCount = 0

def invalidObjectsCount():
    return errorObjectsCount

def run(workDirectory, data, node):
    global log
    global errorObjectsCount
    log = logger.getUploadLogger("Node", node.id)
    global format
    errorList = []
    format = node.currentConverter.format
    global workDir
    workDir = workDirectory
    global packSize
    packSize = node.pack_size
    global charset
    cf = CatalogFactory()
    charset = node.currentConverter.charset

    log.info(logInfo + u"Старт конвертации")

    listDefaultTables = ['classif', 'clasdisc', 'claslim', 'classtax', 'depart', 'longbar', 'bar', 'cardrate', 'cliclass', 'clients', 'credcard',
                         'credpref', 'curs', 'dclislst', 'disccard', 'disccli', 'discsum', 'personal', 'plucash', 'pludisc', 'plugdisc', 'plulim',
                         'plutax', 'pricekin', 'scales', 'sizes', 'tax', 'properties', 'properties_values', 'plu_properties_values']

    action = os.path.splitext(node.currentFlag.lower())[1][1:]
    if not action in ["upd", "cng"]:
        raise AttributeError("In exchange format SUPERMAG flag monitor should be 'cng', or 'upd'")

    tablesNameLoadedIntoMemory = ["classif", "plu_properties_values", "plulim", "claslim", "bar", "longbar", "dclislst"]
    tablesInMemory = {}
    readers = {}
    if (format == 'dat'):
        tablesInMemory, readers = readDATFiles(data, listDefaultTables, tablesNameLoadedIntoMemory)
    else:
        log.error(u"Поддерживается только dat формат")

    groups = findParentGroups(tablesInMemory)

    groupsWithParents = {}
    for g in groups:
        groupsWithParents[g] = groups[g][2]
    cf.setParams(node.currentConverter, groupsWithParents)

    result = False
    try:
        plugin = __import__('upload.inserter.%s' % (node.inserter.name), fromlist='Inserter')
        cl = getattr(plugin, 'Inserter')
        inserter = cl(node)
        result = inserter.start()
        if result:
            result = False

            parseTmc(tablesInMemory, readers, action, inserter, groups, cf)
            parseGroups(tablesInMemory, action, inserter)
            parseBarcodes(tablesInMemory, action, inserter)
            parseAdditionallyPrices(readers, action, inserter)
            parseDept(readers, action, inserter)
            parseDiscCard(tablesInMemory, readers, action, inserter, node.currentConverter.discount)
            personal(readers, action, inserter, node)

            result = inserter.stop()
            errorObjectsCount = inserter.invalidObjectsCount()
    except InserterError as e:
        raise
    except Exception as e:
        log.exception(u'Ошибка: ' + e)
        errorList.append(u'Ошибка: ' + e)

    log.info(logInfo + u"Конвертация завершена")
    return result, errorList

def readDATFiles(data, listDefaultTables, tablesNameLoadedIntoMemory):
    currentTables = {}
    readers = {}
    for dictFile in data:
        fileExtension = os.path.split(dictFile)[1]
        tableName = fileExtension.split('.')[0].lower()

        if (tableName in listDefaultTables):
            log.info(logInfo + "чтение %s" % (tableName))
            reader = csv.reader(open(dictFile, 'rb'), delimiter=',', quotechar='"')
            if tableName in tablesNameLoadedIntoMemory:
                currentTables[tableName] = list(reader)
            else:
                readers[tableName] = reader
        else:
            log.error(logInfo + u"Неизвестная таблица '%s'" % tableName)

    return currentTables, readers

def findParentGroups(tables):
    groups = {}
    if not "classif" in tables:
        return groups

    for group in tables.get("classif"):
        groups[group[0]] = [group[1], group[2]]

    for groupCode in groups:
        parents = []
        tempGroupCode = groupCode
        while tempGroupCode != '' and not tempGroupCode in parents:
            parents.append(tempGroupCode)
            tempGroupCode = groups[tempGroupCode][0]
        groups[groupCode].append(parents)

    return groups

def parseGroups(tables, action, inserter):
    log.info(logInfo + u'Парсинг групп товаров')
    if not "classif" in tables:
        return

    log.info(logInfo + u'Создание объекта группы товаров')
    listObjects = []
    if (action == "cng"):
        listObjects.append({"command": "clearInventGroup"})

    for cl in tables["classif"]:
        listObjects.append(aif.InventGroup().addCmd(groupcode=cl[0], parentgroupcode=cl[1], groupname=encode(cl[2])))
    inserter.add_package(listObjects)

def parseBarcodes(tables, action, inserter):
    log.info(logInfo + u'парсинг штрихкодов')

    tableBarcode = getBarcodeTableName(tables)
    if tableBarcode == '':
        return

    listObjects = []
    if (action == "cng"):
        listObjects.append({"command": "clearBarcode"})

    for barcode in tables.get(tableBarcode, []):
        cquant = barcode[3]
        if cquant == 0:
            cquant = 1
        listObjects.append(aif.Barcode().addCmd(code=encode(barcode[1]), barcode=barcode[0], price=0, name=None, cquant=cquant, measure=None))

    inserter.add_package(listObjects)

def parseTmc(tables, readers, action, inserter, groups, cf):
    log.info(logInfo + u'Парсинг tmc')

    if not "plucash" in readers:
        return

    log.info(logInfo + u'Создание объекта tmc')
    listObjects = []
    if (action == "cng"):
        listObjects.append({"command": "clearInventory"})
        listObjects.append({"command": "clearUnit"})
        listObjects.append({"command": "clearPrice"})

    pluPropVal = getDictPluPropValues(tables.get("plu_properties_values", []))
    pluLim = listToDict(tables.get("plulim", []))
    clasLim = listToDict(tables.get("claslim", []))

    tableBarcodeName = getBarcodeTableName(tables)
    barcodes = {}
    if tableBarcodeName != '':
        barcodes = getDictBarcodes(tables.get(tableBarcodeName, []))

    options = {}
    options['priceoptions'] = {}
    options['priceoptions']['requiredeferredprice'] = 1
    measure = {}
    obj = {'invent': {"inventcode": None}}
    for inv in readers.get('plucash', []):
        inventCode = encode(inv[0])
        extendedoptions = ''
        if "plu_properties_values" in tables:
            for prop in pluPropVal.get(inventCode, {}):
                extendedoptions += encode(prop[2]) + ','
            if extendedoptions != '':
                extendedoptions = extendedoptions[:-1]
            else:
                extendedoptions = None

        groupCode = inv[5]

        if not inv[2] in measure.keys():
            measure[inv[2]] = crc16(inv[2])

        obj = aif.Invent().addCmd(inventcode=inventCode, barcode = None, name=encode(inv[1]), price=0, deptcode=inv[8], cquant=barcodes.get(inventCode, 1),
                                  measurecode=measure[inv[2]], inventgroup=groupCode, options = options, minprice=0, taxgroupcode=None,
                                  extendedoptions=extendedoptions)
        listObjects.append(obj)

        cf.addInventCatalog(groupCode, inventCode)

        minPrice = getMinPrice(inventCode, inv[6], inv[5], pluLim, clasLim, groups)
        listObjects.append(aif.Price().addCmd(barcode = encode(inv[0]), price = inv[6], documentid = '',
            effectivedate = '1970-01-02 00:00:00', doctype = 1, minprice = minPrice))

        if (len(listObjects) >= packSize):
            inserter.add_package(listObjects)
            del listObjects[:]

    for me in measure:
        fractional = True
        if (inv[3] == '0' or inv[3] == '1'):
            fractional = False
        listObjects.append({"command":"addUnit", "unit":{"unitcode":measure[me], "name":encode(me), "fractional":fractional}})
    inserter.add_package(listObjects)

    inserter.add_package(cf.getAifObjects())

# Дополнительные цены и скидки на количество
def parseAdditionallyPrices(readers, action, inserter):
    if not "disccard" in readers:
        return

    listObjects = []
    if action == "cng":
        listObjects.append({"command": "clearAdditionalPrice"})

    for dis in readers.get("disccard", []):
        listObjects.append(aif.AdditionalPrice().addCmd(barcode=encode(dis[0]), pricecode=dis[1], price=dis[3]))

    inserter.add_package(listObjects)

def parseDept(readers, action, inserter):
    if not "clients" in readers:
        return

    log.info(logInfo + u'Создание объекта отдел')
    listObjects = []
    if (action == "cng"):
        listObjects.append({"command": "clearDept"})

    for cl in readers.get('clients', []):
        if (cl[12] == '2'):
            obj = aif.Dept().addCmd(code=cl[0], name=encode(cl[2]))
            listObjects.append(obj)
            if (len(listObjects) == packSize):
                inserter.add_package(listObjects)
    if len(listObjects) > 0:
        inserter.add_package(listObjects)

def parseDiscCard(tables, readers, action, inserter, discount):
    if not "disccli" in readers:
        return

    if not "dclislst" in tables:
        log.error(logInfo + u'Таблица dclislst в smag.sql не найдена')
        return

    MODERN = 'modern'
    LEGACY = 'legacy'

    if not discount in [MODERN, LEGACY]:
        raise AttributeError(u"В обмене в формате SUPERMAG дисконт должен быть '%s' или '%s'" % (LEGACY, MODERN))

    if discount == LEGACY:
        discLegacy(tables, readers, action, inserter)
    elif discount == MODERN:
        discModern(tables, readers, action, inserter)

def discModern(tables, readers, action, inserter):
    log.info(logInfo + u'Новая дисконтная система. Создание Card, CardGroup.')

    listObjects = []
    if (action == "cng"):
        listObjects.append({"command": "clearCard"})
        listObjects.append({"command": "clearCardGroup"})

    for card in readers.get('disccli', []):
        if card[0]:
            if card[0][0] != '!':
                denyCard = 0
                if (isBlockedCard(tables.get('dclislst', []), card[0])):
                    denyCard = 1
                obj = aif.Card().addCmd(
                              idcard = card[0],
                              idcardgroup = 1,
                              number = card[0],
                              blocked = denyCard,
                              )
                listObjects.append(obj)
                if (len(listObjects) >= packSize):
                    inserter.add_package(listObjects)
                    listObjects = []

    if (len(listObjects) > 0):
        inserter.add_package(listObjects)

def discLegacy(tables, readers, action, inserter):
    log.info(logInfo + u'Старая дисконтная система. Создание LDiscCard, LDiscAccumulate.')
    listObjects = []
    if (action == "cng"):
        listObjects.append({"command": "clearLDiscCard"})
        listObjects.append({"command": "clearLDiscAccumulate"})

    for card in readers.get('disccli', []):
        denyCard = 0
        if (isBlockedCard(tables.get('dclislst'), card[0])):
            denyCard = 1

        obj = aif.LDiscCard().addCmd(cardnumber=card[0], name=card[1], rtext=card[0], denycard=denyCard, acdiscscheme=card[2] * 100, cardmode=0)
        listObjects.append(obj)
        if (len(listObjects) == packSize):
            inserter.add_package(listObjects)
            listObjects = []

    if len(listObjects) > 0:
        inserter.add_package(listObjects)

def personal(readers, action, inserter, node):
    log.info(logInfo + u'Добавление пользователей')

    if not "personal" in readers:
        return

    listObjects = []
    if (action == "cng"):
        listObjects.append({"command": "clearMCashUser"})

    for person in readers.get("personal", []):
        obj = {"command":"addMCashUser",
               "mcashuser":{
                            "code":person[0],
                            "name":encode(person[1]),
                            "login":person[0],
                            "password":person[2],
                            "roleusers":[
                                         {
                                          "rolecode":str(node.currentConverter.defaultRoleCode),
                                          "rule":"1"
                                          }
                                         ]
                            }
               }
        if node.currentConverter.relationRolesBy == 'name':
            if node.currentConverter.substringInAdminName in str(encode(person[1])):
                obj["mcashuser"]["roleusers"].append({"rolecode":node.currentConverter.additionallyRoleCode,"rule":"1"})
        else:
            if person[0] in node.currentConverter.usersWithAdditionallyRole:
                obj["mcashuser"]["roleusers"].append({"rolecode":node.currentConverter.additionallyRoleCode,"rule":"1"})

        listObjects.append(obj)

    packSize = len(listObjects)
    if packSize:
        addSmallPackage(inserter, listObjects, packSize)

def addSmallPackage(inserter, listObjects, sizePackage):
    sizePackage = packSize
    i = 0
    while i <= len(listObjects) / sizePackage:
        inserter.add_package(listObjects[i*sizePackage:(i+1)*sizePackage])
        i += 1

        listObjects = []
    return True

def getMinPrice(inventCode, price, groupCode, pluLim, clasLim, groups):
    minPrice = 0.0
    percent = 0.0
    if inventCode in pluLim:
        percent = float(pluLim[inventCode])
    else:
        if groupCode in groups:
            for group in groups.get(groupCode)[2]:
                if group in clasLim:
                    percent = float(clasLim.get(group))
                    break

    if percent != 0.0 and str(price):
        minPrice = float(float(float(100 - percent) * float(price)) / 100)
    return minPrice

def getBarcodeTableName(tables):
    tableBarcode = ''
    if not "bar" in tables:
        if not "longbar"in tables:
            return ''
        else:
            tableBarcode = "longbar"
    else:
        tableBarcode = "bar"
    return tableBarcode

def getDictPluPropValues(pluPropValues):
    pluPropValuesDict = {}
    for pluProp in pluPropValues:
        pluPropValuesDict[pluProp[1]] = []

    for pluProp in pluPropValues:
        pluPropValuesDict[pluProp[1]].append([pluProp[0], pluProp[2], pluProp[3]])

    return pluPropValuesDict

def getDictBarcodes(barcodes):
    barcodesDict = {}
    for bar in barcodes:
        if not bar[1] in barcodesDict:
            barcodesDict[bar[1]] = bar[3]
    return barcodesDict

def listToDict(listValues):
    dictionaries = {}
    for item in listValues:
        dictionaries[item[0]] = item[1]
    return dictionaries


def isBlockedCard(blockedCardTable, cardCode):
    blocked = False
    for block in blockedCardTable:
        if block[0] == cardCode:
            blocked = True
            break
    return blocked

def encode(str):
    result = str
    if type(str) == StringType:
        result = unicode(str, charset)
        result = result.encode('utf-8')
    return result